! *****************************************************************
!
! This file contains a Fortran 90 module for computing the differences 
! between the analytic and numerical solutions
!
! Copyright (C) 2009 David McInerney, University of Chicago
!
! *****************************************************************
!
!    This program is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with this program (see file "COPYING").  If not, see
!    <http://www.gnu.org/licenses/>.
!
! *****************************************************************


module differences

! module for calculating and printing differences between numerical 
! and analytic solutions

  use grid, only : JJ, NN, t, delta_t, KK, iteration
  use analytic, only : analytic_zeta_at_zeta, analytic_u_at_zeta, &
       analytic_v_at_zeta

  implicit none

  double precision zeta_diff_sum, zeta_anal_sum, q_diff_sum, q_anal_sum

contains

! *******************************************************************
! reset sums of differences to zero

  subroutine reset_differences

    implicit none

    zeta_diff_sum = 0d0
    zeta_anal_sum = 0d0
    q_diff_sum = 0d0
    q_anal_sum = 0d0
    
  end subroutine reset_differences

! *******************************************************************
! calculate differences between numerical and analytic solution over the 
! area of the lake, at a single time

  subroutine numerical_differences

    use grid, only : zeta, u_new, v_new

    implicit none

    integer j, k

       do j=1,JJ
          do k=1,KK
             zeta_diff_sum = zeta_diff_sum  &
                  + abs(zeta(j,k)-analytic_zeta_at_zeta(j,k,t))
             zeta_anal_sum = zeta_anal_sum &
                  + abs(analytic_zeta_at_zeta(j,k,t))  
             q_diff_sum = q_diff_sum &
                  + sqrt(((u_new(j-1,k)+u_new(j,k))/2d0 &
                      -analytic_u_at_zeta(j,k,t))**2 &
                  + ((v_new(j,k-1)+v_new(j,k))/2d0 &
                      -analytic_v_at_zeta(j,k,t))**2)
             q_anal_sum = q_anal_sum &
                  + sqrt(analytic_u_at_zeta(j,k,t)**2 &
                         +analytic_v_at_zeta(j,k,t)**2)
          end do
       end do

  end subroutine numerical_differences

! ****************************************************************************
! print differences that have been calculated over a number of iterations

  subroutine print_differences

    implicit none

    print*
    print*,"After ",iteration/NN," periods"
    print*,"differences between numerical and 2nd order solution are"
    print 10,zeta_diff_sum/zeta_anal_sum*100.0
    print 50,q_diff_sum/q_anal_sum*100.0

    write(1001,*)
    write(1001,*)"After ",iteration/NN," periods"
    write(1001,*)"differences between numerical and 2nd order solution are"
    write(1001,10)zeta_diff_sum/zeta_anal_sum*100.0
    write(1001,50)q_diff_sum/q_anal_sum*100.0
    flush(1001)
    
10  format ("      ",F12.8,"% for zeta")
50  format ("      ",F12.8,"% for q")

  end subroutine print_differences

! ****************************************************************************

end module differences
